package infoh509.tp3.jdom.ex2;

import java.io.File;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.jdom.Document;
import org.jdom.Element;
import org.jdom.input.SAXBuilder;
import org.jdom.output.XMLOutputter;

public class MergeCustomers {
	/**
	 * Add every customers in list to the specified Map. Optionally add customers in the
	 * target document.
	 * 
	 * @param customers A list of Customer elements which must be added to the map
	 * @param customersList The map to be fille with customers
	 * @param target An XML Element whose content is to be replaced with customers.
	 */
	@SuppressWarnings("unchecked")
	public static void addCustomers(List customers, Map<String, Element> customersList, Element target) {
		for(Object c: customers) {
			Element customer = (Element)c;
			String customerID = customer.getAttributeValue("CustomerID");
			
			if (target != null) {
				/* Elements must be cloned when going from a document to another */
				Element clone = (Element)customer.clone();
				
				/*If there is already a customer, replace it in the document,
				 * otherwise add it. */
				if (customersList.containsKey(customerID)) {
					Element originalCustomer = customersList.get(customerID);
					
					int index = target.indexOf(originalCustomer);
					target.setContent(index, clone);
				}
				else {
					target.addContent(clone);
				}
			}
			
			customersList.put(customerID, customer);
		}
	}
	
	public static void main(String[] args) {
		Map<String, Element> customersList = new HashMap<String, Element>(); 
		
		try {
			SAXBuilder b = new SAXBuilder();
			Document docCustomers = b.build(new File("data/customers.xml"));
			Document docOrders = b.build(new File("data/orders.xml"));
			
			/* The element which will contain the new list of customers */
			Element target = docCustomers.getRootElement();
			
			/* Register each customer from customers.xml in the map */
			addCustomers(target.getChildren("Customer"), 
					customersList, null);
			
			/* Add customers from orders.xml */
			addCustomers(docOrders.getRootElement().getChild("Customers").
					getChildren("Customer"), customersList, target);
			
			/* Show the result */
			XMLOutputter outputter = new XMLOutputter();
			outputter.output(docCustomers,System.out);
		} 
		catch (Exception e) { 
			e.printStackTrace(); 
		}
	}
}
