package infoh509.tp3.sax.ex2;

import org.xml.sax.Attributes;
import org.xml.sax.Locator;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.helpers.XMLReaderFactory;

public class PrettyPrint  extends DefaultHandler {
	private int level = 0;     
	private Locator loc = null;
	private int lastLine = 0;
	
	private void printLine(String text) {
		StringBuffer lineBuffer = new StringBuffer();
		
		/* N'affiche le numero de ligne que si on a change de ligne */
		if (loc != null && lastLine != loc.getLineNumber()) {
			lineBuffer.append(loc.getLineNumber());
			lastLine = loc.getLineNumber();
		}
		
		/* utilise une tabulation par niveau d'indentation */
		for (int i = 0; i < level; i++){
			lineBuffer.append('\t');
		}
		
		lineBuffer.append(text);
		System.out.println(lineBuffer.toString());
	}
	
	public void setDocumentLocator(Locator locator) {
		this.loc = locator;
	}
	
	public void startDocument() {
		printLine("<?xml version=\"1.1\" ?>");
	}
	
	public void processingInstruction(String target, String data) {
		printLine("<?" + target + " " + data + "?>");
	}
	
	public void startElement(String uri, String localName,
			String qName, Attributes atts) {

		StringBuffer element = new StringBuffer();
		
		element.append("<");
		element.append(localName); /* dans cet exemple, on oublie les namespaces */
		
		for (int i = 0; i < atts.getLength(); i++) {
			element.append(" ");
			element.append(atts.getLocalName(i));
			element.append("=\"");
			element.append(atts.getValue(i));
			element.append("\"");
		}
		element.append(">");
		
		printLine(element.toString());
		level++;
	}

	public void endElement(String uri, String localName, 
			String qName) {
		level--;
		printLine("<" + localName + "/>");
	}

	public void characters(char[] ch, int start, int length) {
		String text = new String(ch,start,length);
		
		if (text.trim().length() != 0) {
			level++;
			printLine(text);
			level--;
		}
	}

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		XMLReader reader;
		try {
			PrettyPrint myHandler = new PrettyPrint();
			
			/* Read the file passed as parameter using SAX */
			reader = XMLReaderFactory.createXMLReader();
			reader.setContentHandler(myHandler);
			reader.parse(args[0]);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
}
